﻿GameObject = {}

-- Some global fields
GameObject.MainInput = nil
GameObject.MainConsole = nil

local GameState = { Title = 1, Menu = 2, Game = 3 }
local GameMode = { Sprint = 1, Endurance = 2 }
local Difficulty = { Easy = 1, Medium = 2, Hard = 3 }

local gameState = GameState.Title
local gameMode = GameMode.Sprint
local gameDifficulty = Difficulty.Easy
local maxSpeed = 15.0
local wheelAnimation = 0.0
local totalTime = 0.0
local startCountdown = nil
local countdownComplete = false

local leftLegPosition = 0
local rightLegPosition = 0
local leg = 0
local speed = 5.0
local progress = 0.0
local lastLegTime = 0.0
local speedModifier = 0.0
local detailTime = 0.0
local timeRemaining = 5.0
local levelLength = 40.0
local initialTime = 0.0

-- 1 grass, 2 sand, 3 snow
local environment = 1
local levelCompleted = false
local gameActive = false
local menuWindow, optionGameModeLabel, optionDifficultyLabel, optionEnvironmentLabel = nil, nil, nil, nil

GameObject.init = function ()
	print("Init GameObject ...")
	Sound.play(Music)
end

GameObject.draw = function ()

	if gameState == GameState.Game then

		GameObject.drawSky()
		GameObject.drawGround()
		GameObject.drawBicycle()
		Screen.fillRow({y = 0, color = Color.Black})

		if gameActive then

			-- Speed
			Screen.print({x = 0, y = 0, text = "Speed", color = Color.Yellow })
			Screen.print({x = 6, y = 0, text = "▓▓▓▓▓▓▓▓▓▓", color = Color.Gray })
			local speedLevel = 1 + math.floor( ((speed - 5.0) / (maxSpeed - 5.0)) * 10.0 )
			Screen.print({x = 6, y = 0, text = string.sub("          ", 1, speedLevel), background = Color.Yellow })
			Screen.print({x = 0, y = 24, text = "S", color = Color.Yellow, background = Color.Red })
			Screen.print({x = 79, y = 24, text = "F", color = Color.Yellow, background = Color.Red })
			local dotPosition = 1 + math.floor( (progress / levelLength) * 78.0 )
			Screen.print({x = dotPosition, y = 24, text = "▄", color = Color.LightRed })

			-- Timer
			Screen.print({x = 37, y = 0, text = stringFromTime(timeRemaining), color = Color.Yellow })

		end

		if countdownComplete == false then
			Screen.drawBorder({width = 28, height = 5, x = 26, y = 10, color = Color.LightBlue, shadow = true, background = Color.Blue, fill = true })
			local startCountdownSecond = math.ceil(startCountdown)
			if startCountdownSecond == 3 then
				Screen.print({x = 34, y = 12, text = "ON YOUR MARK", color = Color.Yellow })
			elseif startCountdownSecond == 2 then
				Screen.print({x = 37, y = 12, text = "GET SET", color = Color.Yellow })
			elseif startCountdownSecond == 1 then
				Screen.print({x = 39, y = 12, text = "GO!", color = Color.Yellow })
			end
		end

	elseif gameState == GameState.Title then

		Screen.drawBorder({width = 80, height = 25, color = Color.Cyan})

		local shadowColor = Color.Gray
		Screen.print({x = 14, y = 2, text = "      ╗   ╗       ╗  ╗     ╗       ╗  ╗            ╗", color = shadowColor, raw = true})
		Screen.print({x = 14, y = 3, text = "  ╔══  ╗  ║  ╔════╝╚  ╗   ╔╝  ╔════╝  ║       ╔════╝", color = shadowColor, raw = true})
		Screen.print({x = 14, y = 4, text = "      ╔╝  ║  ║      ╚    ╔╝   ║       ║          ╗", color = shadowColor, raw = true})
		Screen.print({x = 14, y = 5, text = "  ╔══  ╗  ║  ║       ╚  ╔╝    ║       ║       ╔══╝", color = shadowColor, raw = true})
		Screen.print({x = 14, y = 6, text = "      ╔╝  ║╚      ╗     ║   ╚      ╗       ╗       ╗", color = shadowColor, raw = true})
		Screen.print({x = 14, y = 7, text = "╚═════╝ ╚═╝ ╚═════╝   ╚═╝    ╚═════╝╚══════╝╚══════╝", color = shadowColor, raw = true})
		Screen.print({x = 4, y = 9, text = "       ╗  ╗   ╗      ╗  ╗     ╗  ╗           ╗         ╗       ╗       ╗", color = shadowColor, raw = true})
		Screen.print({x = 4, y = 10, text = "  ╔════╝  ║    ╗     ║  ║     ║  ║       ╔══  ╗╚══  ╔══╝  ╔═══  ╗  ╔══  ╗", color = shadowColor, raw = true})
		Screen.print({x = 4, y = 11, text = "       ╗  ║  ╔    ╔  ║  ║     ║  ║            ║     ║     ║     ║      ╔╝", color = shadowColor, raw = true})
		Screen.print({x = 4, y = 12, text = "╚════  ║  ║  ║╚  ╔╝  ║  ║     ║  ║       ╔══  ║     ║     ║     ║  ╔══  ╗", color = shadowColor, raw = true})
		Screen.print({x = 4, y = 13, text = "       ║  ║  ║ ╚═╝   ║╚      ╔╝       ╗  ║    ║     ║   ╚      ╔╝  ║    ║", color = shadowColor, raw = true})
		Screen.print({x = 4, y = 14, text = "╚══════╝╚═╝╚═╝     ╚═╝ ╚═════╝ ╚══════╝╚═╝  ╚═╝   ╚═╝    ╚═════╝ ╚═╝  ╚═╝", color = shadowColor, raw = true})
		Screen.print({x = 25, y = 16, text = "   ╗      ╗       ╗   ╗    ╗", color = shadowColor, raw = true})
		Screen.print({x = 25, y = 17, text = "   ║  ╔══  ╗  ╔══  ╗  ║    ║", color = shadowColor, raw = true})
		Screen.print({x = 25, y = 18, text = "╚  ║╚      ║╚     ╔╝       ║", color = shadowColor, raw = true})
		Screen.print({x = 25, y = 19, text = "   ║ ╚═══  ║  ╔══  ╗╚════  ║", color = shadowColor, raw = true})
		Screen.print({x = 25, y = 20, text = "   ║      ╔╝╚     ╔╝       ║", color = shadowColor, raw = true})
		Screen.print({x = 25, y = 21, text = " ╚═╝ ╚════╝  ╚════╝      ╚═╝", color = shadowColor, raw = true})

		local titleColor = Color.White
		Screen.print({x = 14, y = 2, text = "██████  ██  ██████ ██    ██  ██████ ██      ███████ ", color = titleColor, raw = true})
		Screen.print({x = 14, y = 3, text = "██   ██ ██ ██       ██  ██  ██      ██      ██      ", color = titleColor, raw = true})
		Screen.print({x = 14, y = 4, text = "██████  ██ ██        ████   ██      ██      █████ ", color = titleColor, raw = true})
		Screen.print({x = 14, y = 5, text = "██   ██ ██ ██         ██    ██      ██      ██    ", color = titleColor, raw = true})
		Screen.print({x = 14, y = 6, text = "██████  ██  ██████    ██     ██████ ███████ ███████ ", color = titleColor, raw = true})
		Screen.print({x = 14, y = 7, text = "                                                    ", color = titleColor, raw = true})
		Screen.print({x = 4, y = 9, text = "███████ ██ ███    ███ ██    ██ ██       █████  ████████  ██████  ██████ ", color = titleColor, raw = true})
		Screen.print({x = 4, y = 10, text = "██      ██ ████  ████ ██    ██ ██      ██   ██    ██    ██    ██ ██   ██ ", color = titleColor, raw = true})
		Screen.print({x = 4, y = 11, text = "███████ ██ ██ ████ ██ ██    ██ ██      ███████    ██    ██    ██ ██████  ", color = titleColor, raw = true})
		Screen.print({x = 4, y = 12, text = "     ██ ██ ██  ██  ██ ██    ██ ██      ██   ██    ██    ██    ██ ██   ██ ", color = titleColor, raw = true})
		Screen.print({x = 4, y = 13, text = "███████ ██ ██      ██  ██████  ███████ ██   ██    ██     ██████  ██   ██ ", color = titleColor, raw = true})
		Screen.print({x = 4, y = 14, text = "                                                                         ", color = titleColor, raw = true})
		Screen.print({x = 25, y = 16, text = " ██  █████   █████  ██   ██ ", color = titleColor, raw = true})
		Screen.print({x = 25, y = 17, text = "███ ██   ██ ██   ██ ██   ██ ", color = titleColor, raw = true})
		Screen.print({x = 25, y = 18, text = " ██  ██████  █████  ███████ ", color = titleColor, raw = true})
		Screen.print({x = 25, y = 19, text = " ██      ██ ██   ██      ██ ", color = titleColor, raw = true})
		Screen.print({x = 25, y = 20, text = " ██  █████   █████       ██ ", color = titleColor, raw = true})
		Screen.print({x = 25, y = 21, text = "                            ", color = titleColor, raw = true})

		Screen.print({ x = 28, y = 24, text = " Press RETURN to begin ", color = Color.Yellow})

	elseif gameState == GameState.Menu then
		Screen.drawBorder({width = 80, height = 25, color = Color.LightBlue })
		Screen.print({ x = 29, y = 22, text = "Created by @EIGENLENK", color = Color.Yellow})
		Screen.print({ x = 28, y = 3, text = "Bicycle Simulator 1984", color = Color.Yellow})
		Screen.print({ x = 37, y = 4, text = "LD34", color = Color.Brown})
	end

end

GameObject.update = function (frameTime)

	totalTime = totalTime + frameTime

	Screen.redraw()

	if gameState == GameState.Game then

		if countdownComplete == false then
			startCountdown = startCountdown - frameTime
			if startCountdown < 0.0 then
				countdownComplete = true
			end
		end

		wheelAnimation = wheelAnimation + (speed * 3.5 * frameTime)

		if wheelAnimation > 10.0 then
			wheelAnimation = 0
		end

		if levelCompleted == false and gameActive == true and countdownComplete then
			progress = progress + (speed * frameTime * 0.1)
			if progress >= levelLength then
				progress = levelLength
				GameObject.levelCompleted()
			end
		end

		if speedModifier > 0.1 then
			local accel = (maxSpeed - speed) * 0.4
			speed = speed + (speedModifier * frameTime * accel)
		else
			speedModifier = 0.0
			speed = math.max(speed - ((speed * frameTime) * 0.25), 5.0)
		end

		speedModifier = speedModifier - ((speedModifier * frameTime) * 1.50)
		detailTime = detailTime + ((speed * (speed / 16)) * 6 * frameTime)

		if gameActive and countdownComplete then
			timeRemaining = timeRemaining - frameTime
			if timeRemaining <= 1.0 then
				timeRemaining = 0.0
				GameObject.levelFailed()
			end
		end
	end

end

GameObject.onKey = function (keyCode, pressed, handled)

	-- print("Key code:", keyCode, "Pressed:", pressed, "Handled:", handled)

	-- ESC
	if keyCode == 36 and pressed and handled == false then
		if gameState == GameState.Game then
			GameObject.goToMenu()
		else
			Core.exit(0)
		end
	elseif (keyCode == 71 or keyCode == 17) and pressed and handled == false and leg == 0 and gameActive and countdownComplete then
		if leftLegPosition == 0 and rightLegPosition == 0 then
			leftLegPosition = 1
			rightLegPosition = 1
			GameObject.calculateSpeed()
			leg = 1
		elseif leftLegPosition == 2 and rightLegPosition == 2 then
			leftLegPosition = 3
			rightLegPosition = 3
			GameObject.calculateSpeed()
			leg = 1
		end
	elseif (keyCode == 72 or keyCode == 11) and pressed and handled == false and leg == 1 and gameActive and countdownComplete then
		if rightLegPosition == 1 and leftLegPosition == 1 then
			leftLegPosition = 2
			rightLegPosition = 2
			GameObject.calculateSpeed()
			leg = 0
		elseif rightLegPosition == 3 and leftLegPosition == 3 then
			leftLegPosition = 0
			rightLegPosition = 0
			GameObject.calculateSpeed()
			leg = 0
		end
	-- Switch between display styles
	elseif (keyCode == 86 or keyCode == 3) and pressed then
		if Core.getDisplayStyle() == DisplayStyle.Default then
			Core.setDisplayStyle(DisplayStyle.FullFill)
		elseif Core.getDisplayStyle() == DisplayStyle.FullFill then
			Core.setDisplayStyle(DisplayStyle.FullFit)
		else
			Core.setDisplayStyle(DisplayStyle.Default)
		end

	-- Toggle fullscreen
	elseif (keyCode == 87 or keyCode == 5) and pressed then
		Core.toggleFullscreen()

	-- Cycle wallpaper
	elseif (keyCode == 88 or keyCode == 22) and pressed then
		GameObject.cycleWallpaper()

	-- Return
	elseif keyCode == 58 and pressed and handled == false then
		if gameState == GameState.Title then
			GameObject.goToMenu()
		elseif gameState == GameState.Menu then
			GameObject.beginGame()
		end
	end

end

GameObject.drawSky = function()
	Screen.fillRect({ y = 0, color = Color.LightBlue, width = 80, height = 3 })
	Screen.fillRect({ y = 3, color = Color.LightCyan, width = 80, height = 6 })
	Screen.print({ x = 0, y = 3, color = Color.LightBlue, text = "▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓"})
	Screen.print({ x = 0, y = 4, color = Color.LightBlue, text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░"})
	Screen.print({ x = 0, y = 7, color = Color.White, text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░"})
	Screen.print({ x = 0, y = 8, color = Color.White, text = "▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓"})
	Screen.print({ x = 0, y = 1, color = Color.Blue, text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░"})
end

GameObject.drawGround = function()

	local groundColor1 = nil
	local groundColor2 = Color.Gray
	local groundColor3 = nil

	-- Grassy
	if environment == 1 then
		groundColor1 = Color.LightGreen
		groundColor3 = Color.Green
		Screen.print({ x = 5, y = 8, color = groundColor1,	text = "███▓", raw = true, background = groundColor3 })
		Screen.print({ x = 53, y = 8, color = groundColor1,	text = "████████▓", raw = true, background = groundColor3 })
		Screen.print({ x = 6, y = 7, color = groundColor3,	text = "▲▲", raw = true, background = groundColor1 })
		Screen.print({ x = 0, y = 8, color = groundColor3,	text = "▲ ▲▲", raw = true, background = groundColor1 })
		Screen.print({ x = 16, y = 8, color = groundColor3,	text = "▲ ▲▲ ▲  ▲", raw = true, background = groundColor1 })
		Screen.print({ x = 55, y = 7, color = groundColor3,	text = "▲▲  ▲", raw = true, background = groundColor1 })
		Screen.print({ x = 67, y = 8, color = groundColor3,	text = "▲  ▲▲▲ ▲  ▲", raw = true, background = groundColor1 })

	-- Desert
	elseif environment == 2 then
		groundColor1 = Color.Yellow
		groundColor3 = Color.Brown
		Screen.print({ x = 5, y = 7, color = Color.Yellow,	text = " █▓ ", raw = true, background = Color.Brown })
		Screen.print({ x = 5, y = 8, color = Color.Yellow,	text = "███▓", raw = true, background = Color.Brown })
		Screen.print({ x = 50, y = 7, color = Color.Yellow,	text = " █▓ ", raw = true, background = Color.Brown })
		Screen.print({ x = 50, y = 8, color = Color.Yellow,	text = "███▓", raw = true, background = Color.Brown })
		Screen.print({ x = 72, y = 7, color = Color.Yellow,	text = " █ ", raw = true, background = Color.Brown })
		Screen.print({ x = 72, y = 8, color = Color.Yellow,	text = "██▓", raw = true, background = Color.Brown })

	-- Snow
	elseif environment == 3 then
		groundColor1 = Color.White
		groundColor3 = Color.LightGray

		Screen.print({ x = 3, y = 7, color = Color.White,	text = " ███████▓ ", raw = true, background = Color.LightGray })
		Screen.print({ x = 3, y = 8, color = Color.White,	text = "█████████▓", raw = true, background = Color.LightGray })
		Screen.print({ x = 24, y = 8, color = Color.White,	text = "█████▓", raw = true, background = Color.LightGray })
		Screen.print({ x = 70, y = 7, color = Color.White,	text = " ▓████████", raw = true, background = Color.LightCyan })
		Screen.print({ x = 70, y = 8, color = Color.White,	text = "▓██████████", raw = true, background = Color.LightGray })
	end


	Screen.print({ x = 0, y = 9, color = groundColor1, 		text = "████████████████████████████████▀              ▀████████████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 10, color = groundColor1, 	text = "███████████████████████████████▀                ▀███████████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 11, color = groundColor1,		text = "██████████████████████████████▀                  ▀██████████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 12, color = groundColor1,		text = "█████████████████████████████▀                    ▀█████████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 13, color = groundColor1,		text = "████████████████████████████▀                      ▀████████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 14, color = groundColor1,		text = "███████████████████████████▀                        ▀███████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 15, color = groundColor1,		text = "██████████████████████████▀                          ▀██████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 16, color = groundColor1,		text = "█████████████████████████▀                            ▀█████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 17, color = groundColor1,		text = "████████████████████████▀                              ▀████████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 18, color = groundColor1,		text = "███████████████████████▀                                ▀███████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 19, color = groundColor1,		text = "██████████████████████▀                                  ▀██████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 20, color = groundColor1,		text = "█████████████████████▀                                    ▀█████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 21, color = groundColor1,		text = "████████████████████▀                                      ▀████████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 22, color = groundColor1,		text = "███████████████████▀                                        ▀███████████████████", background = groundColor2, raw = true})
	Screen.print({ x = 0, y = 23, color = groundColor1,		text = "██████████████████▀                                          ▀██████████████████", background = groundColor2, raw = true})

	local detail = math.floor(detailTime / 3) % 3

	if detail == 0 then
		Screen.print({ x = 0, y = 9, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 1 then
		Screen.print({ x = 0, y = 10, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                  ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 2 then
		Screen.print({ x = 0, y = 11, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                    ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 0 then
		Screen.print({ x = 0, y = 12, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░░                      ░░░░░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 1 then
		Screen.print({ x = 0, y = 13, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░░                        ░░░░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 2 then
		Screen.print({ x = 0, y = 14, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░░░                          ░░░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 0 then
		Screen.print({ x = 0, y = 15, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░░                            ░░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 1 then
		Screen.print({ x = 0, y = 16, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░░                              ░░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 2 then
		Screen.print({ x = 0, y = 17, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░░                                ░░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 0 then
		Screen.print({ x = 0, y = 18, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░░                                  ░░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 1 then
		Screen.print({ x = 0, y = 19, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░░                                    ░░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 2 then
		Screen.print({ x = 0, y = 20, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░░                                      ░░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 0 then
		Screen.print({ x = 0, y = 21, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░░                                        ░░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 1 then
		Screen.print({ x = 0, y = 22, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░░                                          ░░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end
	if detail == 2 then
		Screen.print({ x = 0, y = 23, color = groundColor3,	text = "░░░░░░░░░░░░░░░░░░                                            ░░░░░░░░░░░░░░░░░░", background = groundColor1, raw = true})
	end

	if detail == 0 then
		Screen.put({ x = 40, y = 9, value = "░", color = Color.Yellow })
	elseif detail == 1 then
		Screen.put({ x = 40, y = 10, value = "░", color = Color.Yellow })
	elseif detail == 2 then
		Screen.put({ x = 40, y = 11, value = "░", color = Color.Yellow })
	end

end

GameObject.drawBicycle = function()

	local bodyColor = Color.LightBlue
	local legColor = Color.Brown
	local bikeColor = Color.White
	local shoeColor = Color.Red
	local startY = 12

	Screen.print({ x = 38, y = startY, color = Color.Gray, text = "▄███▄"})
	Screen.print({ x = 38, y = startY+1, color = Color.Gray, text = "▀   ▀", raw = true})
	Screen.print({ x = 39, y = startY+1, color = Color.Brown, background = Color.LightRed, text = "▀▀▀"})
	Screen.print({ x = 38, y = startY+2, color = bodyColor, text = "▄███▄"})
	Screen.print({ x = 37, y = startY+3, color = bodyColor, text = "▄█████▄"})
	Screen.print({ x = 37, y = startY+4, color = bodyColor, text = "███████"})
	Screen.print({ x = 37, y = startY+5, color = legColor, text = "▓█████▓", raw = true})
	Screen.print({ x = 36, y = startY+3, color = Color.White, text = "▄       ▄", raw = true})
	Screen.print({ x = 36, y = startY+4, color = Color.LightRed, text = "▀       ▀", raw = true})

	if gameMode == GameMode.Endurance then
		if (progress >= levelLength * 0.83) then
			Screen.print({ x = 39, y = startY+2, color = bodyColor, text = "▓▓▓", background = Color.Blue})
			Screen.print({ x = 39, y = startY+3, color = bodyColor, text = "▓▓▓", background = Color.Blue})
			Screen.print({ x = 40, y = startY+4, color = bodyColor, text = "▓", background = Color.Blue})
		elseif (progress >= levelLength * 0.64) then
			Screen.print({ x = 39, y = startY+2, color = bodyColor, text = "▓▓▓", background = Color.Blue})
			Screen.print({ x = 40, y = startY+3, color = bodyColor, text = "▓", background = Color.Blue})
		elseif (progress >= levelLength * 0.3) then
			Screen.print({ x = 40, y = startY+2, color = bodyColor, text = "▓", background = Color.Blue})
		end
	else
		if (progress >= levelLength * 0.5) then
			Screen.print({ x = 40, y = startY+2, color = bodyColor, text = "▓", background = Color.Blue})
		end
	end

	Screen.print({ x = 39, y = startY+6, color = Color.Gray, text = "▀▀▀", background = legColor, raw = true }) -- saddle
	Screen.print({ x = 40, y = startY+6, color = Color.Gray, text = "▀", background = Color.LightGray, raw = true }) -- saddle

	if leftLegPosition == 0 then
		Screen.print({ x = 37, y = startY+6, color = legColor, text = "▄█", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+7, color = legColor, text = "██▀", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+8, color = shoeColor, background = legColor, text = "▄▄", raw = true}) -- shoe
		Screen.print({ x = 37, y = startY+9, color = Color.Gray, text = "▀▀", raw = true}) -- pedals
	elseif leftLegPosition == 1 or leftLegPosition == 3  then
		Screen.print({ x = 37, y = startY+6, color = legColor, text = " █", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+7, color = legColor, text = "███", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+8, color = legColor, text = "███", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+9, color = shoeColor, background = legColor, text = "▄▄", raw = true}) -- shoe
		Screen.print({ x = 37, y = startY+10, color = Color.Gray, text = "▀▀", raw = true}) -- pedals
	elseif leftLegPosition == 2 then
		Screen.print({ x = 38, y = startY+6, color = legColor, text = "█", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+7, color = legColor, text = "▄██", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+8, color = legColor, text = "████", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+9, color = legColor, text = "███", raw = true}) -- legs
		Screen.print({ x = 37, y = startY+10, color = shoeColor, background = legColor, text = "▄▄", raw = true}) -- shoe
		Screen.print({ x = 37, y = startY+11, color = Color.Gray, text = "▀▀", raw = true}) -- pedals
	end

	if rightLegPosition == 0 then
		Screen.print({ x = 42, y = startY+6, color = legColor, text = "█", raw = true}) -- legs
		Screen.print({ x = 41, y = startY+7, color = legColor, text = "██▄", raw = true}) -- legs
		Screen.print({ x = 40, y = startY+8, color = legColor, text = "████", raw = true}) -- legs
		Screen.print({ x = 41, y = startY+9, color = legColor, text = "███", raw = true}) -- legs
		Screen.print({ x = 42, y = startY+10, color = shoeColor, background = legColor, text = "▄▄", raw = true}) -- shoe
		Screen.print({ x = 42, y = startY+11, color = Color.Gray, text = "▀▀", raw = true}) -- pedals
	elseif rightLegPosition == 1 or rightLegPosition == 3 then
		Screen.print({ x = 42, y = startY+6, color = legColor, text = "█", raw = true}) -- legs
		Screen.print({ x = 41, y = startY+7, color = legColor, text = "███", raw = true}) -- legs
		Screen.print({ x = 41, y = startY+8, color = legColor, text = "███", raw = true}) -- legs
		Screen.print({ x = 42, y = startY+9, color = shoeColor, background = legColor, text = "▄▄", raw = true}) -- shoe
		Screen.print({ x = 42, y = startY+10, color = Color.Gray, text = "▀▀", raw = true}) -- pedals
	elseif rightLegPosition == 2 then
		Screen.print({ x = 42, y = startY+6, color = legColor, text = "█▄", raw = true}) -- legs
		Screen.print({ x = 41, y = startY+7, color = legColor, text = "▀██", raw = true}) -- legs
		Screen.print({ x = 42, y = startY+8, color =shoeColor, background = legColor, text = "▄▄", raw = true}) -- shoe
		Screen.print({ x = 42, y = startY+9, color = Color.Gray, text = "▀▀", raw = true}) -- pedals
	end

	Screen.print({ x = 40, y = startY+7, color = bikeColor, text = "█"}) -- bike
	Screen.print({ x = 39, y = startY+8, color = bikeColor, text = "▄█▄"}) -- bike
	Screen.print({ x = 39, y = startY+9, color = bikeColor, text = "█ █", raw = true}) -- bike
	Screen.print({ x = 39, y = startY+10, color = bikeColor, text = "▀ ▀", raw = true}) -- bike

	if wheelAnimation < 5.0 then
		Screen.print({ x = 40, y = startY+9, color = Color.Gray, text = "▓"}) -- back wheel
		Screen.print({ x = 40, y = startY+10, color = Color.Gray, text = "█"}) -- back wheel
		Screen.print({ x = 40, y = startY+11, color = Color.Gray, text = "▓"}) -- back wheel
	else
		Screen.print({ x = 40, y = startY+9, color = Color.Gray, text = "█"}) -- back wheel
		Screen.print({ x = 40, y = startY+10, color = Color.Gray, text = "▓"}) -- back wheel
		Screen.print({ x = 40, y = startY+11, color = Color.Gray, text = "█"}) -- back wheel
	end
end

GameObject.calculateSpeed = function()
	local timeDiff = totalTime - lastLegTime
	speedModifier = math.max(1.0 - (timeDiff / 0.14), 0.0)
	lastLegTime = totalTime
end

GameObject.goToMenu = function ()

	if gameActive then
		Sound.play(Music)
	end

	gameState = GameState.Menu

	levelCompleted = false
	gameActive = false

  menuWindow = LuaWindow(UI.createWindow({x = 18, y = 7, width = 44, height = 13, closable = false, shadow = true, color = Color.LightBlue, background = Color.Blue }))

	UI.createLabel({parent = menuWindow.instance(), x = 3, y = 2, text = "1) Game-mode:", color = Color.Yellow})
	optionGameModeLabel = LuaLabel(UI.createLabel({parent = menuWindow.instance(), x = 19, y = 2, color = Color.Yellow, background = Color.Black }))
	UI.createLabel({parent = menuWindow.instance(), x = 3, y = 3, text = "2) Difficulty:", color = Color.Yellow})
	optionDifficultyLabel = LuaLabel(UI.createLabel({parent = menuWindow.instance(), x = 19, y = 3, color = Color.Yellow, background = Color.Black }))
	UI.createLabel({parent = menuWindow.instance(), x = 3, y = 4, text = "3) Environment:", color = Color.Yellow})
	optionEnvironmentLabel = LuaLabel(UI.createLabel({parent = menuWindow.instance(), x = 19, y = 4, color = Color.Yellow, background = Color.Black }))

	if Core.getPlatform() == "windows" then
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 6, text = "F1) Help", color = Color.Yellow })
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 7, text = "F2) Change display mode", color = Color.Yellow })
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 8, text = "F3) Fullscreen", color = Color.Yellow })
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 9, text = "F4) Change wallpaper", color = Color.Yellow })
	elseif Core.getPlatform() == "mac" then
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 6, text = " H) Help", color = Color.Yellow })
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 7, text = " D) Change display mode", color = Color.Yellow })
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 8, text = " F) Fullscreen", color = Color.Yellow })
		UI.createLabel({parent = menuWindow.instance(), x = 2, y = 9, text = " W) Change wallpaper", color = Color.Yellow })
	end

	UI.createLabel({parent = menuWindow.instance(), x = 16, y = 12, text = " [Return ►] ", color = Color.Yellow, background = Color.Blue })

	GameObject.updateMenuLabels()

	menuWindow.onKey(function(keyCode, pressed)
		if pressed then
			if keyCode == 27 then
				if gameMode == GameMode.Sprint then
					gameMode = GameMode.Endurance
				elseif gameMode == GameMode.Endurance then
					gameMode = GameMode.Sprint
				end
				GameObject.updateMenuLabels()
			elseif keyCode == 28 then
				gameDifficulty = gameDifficulty + 1
				if gameDifficulty > 3 then
					gameDifficulty = 1
				end
				GameObject.updateMenuLabels()
			elseif keyCode == 29 then
				environment = environment + 1
				if environment > 3 then
					environment = 1
				end
				GameObject.updateMenuLabels()
			elseif keyCode == 85 or keyCode == 7 then
				GameObject.showHelp()
			end
		end
		return true
	end)
end

GameObject.updateMenuLabels = function()
	if gameMode == GameMode.Sprint then
		optionGameModeLabel.setText(" ◄ Sprint ► ")
	elseif gameMode == GameMode.Endurance then
		optionGameModeLabel.setText(" ◄ Endurance ► ")
	end

	if environment == 1 then
		optionEnvironmentLabel.setText(" ◄ Grasslands ► ")
	elseif environment == 2 then
		optionEnvironmentLabel.setText(" ◄ Desert ► ")
	elseif environment == 3 then
		optionEnvironmentLabel.setText(" ◄ Snowy ► ")
	end

	if gameDifficulty == Difficulty.Easy then
		optionDifficultyLabel.setText(" ◄ Easy ► ")
	elseif gameDifficulty == Difficulty.Medium then
		optionDifficultyLabel.setText(" ◄ Medium ► ")
	elseif gameDifficulty == Difficulty.Hard then
		optionDifficultyLabel.setText(" ◄ Hard ► ")
	end

end

GameObject.beginGame = function()

	menuWindow.close()
	levelCompleted = false
	gameActive = true

	Sound.stop(Music)

	leftLegPosition = 0
	rightLegPosition = 0
	leg = 0
	speed = 5.0
	progress = 0.0
	lastLegTime = 0.0
	speedModifier = 0.0
 	detailTime = 0.0
	startCountdown = 3.0
	countdownComplete = false

	if gameMode == GameMode.Sprint then
		if gameDifficulty == Difficulty.Easy then
			timeRemaining = 20.0
			levelLength = 16.0
		elseif gameDifficulty == Difficulty.Medium then
			timeRemaining = 20.0
			levelLength = 20.0
		elseif gameDifficulty == Difficulty.Hard then
			timeRemaining = 20.0
			levelLength = 23.2
		end
	elseif gameMode == GameMode.Endurance then
		if gameDifficulty == Difficulty.Easy then
			timeRemaining = 180.0
			levelLength = 130.0
		elseif gameDifficulty == Difficulty.Medium then
			timeRemaining = 240.0
			levelLength = 205.0
		elseif gameDifficulty == Difficulty.Hard then
			timeRemaining = 300.0
			levelLength = 300.0
		end
	end

	initialTime = timeRemaining

	gameState = GameState.Game

end

GameObject.levelCompleted = function ()
	levelCompleted = true
	gameActive = false
	print("Level completed in time!")
	Sound.play(Music)
	local window = LuaWindow(UI.createWindow({x = 18, y = 9, width = 44, height = 8, closable = true, shadow = true, color = Color.LightGreen, background = Color.Green, title = "Congratulations!" }))
	UI.createLabel({parent = window.instance(), x = 17, y = 7, text = " [Return ►] ", color = Color.Yellow, background = Color.Green })
	UI.createLabel({parent = window.instance(), x = 9, y = 2, text = "You finished the trial with", color = Color.LightGreen })
	UI.createLabel({parent = window.instance(), x = 7, y = 3, text = "flying colors! Time for a rest.", color = Color.LightGreen })
	local raceTime = initialTime - timeRemaining
	UI.createLabel({parent = window.instance(), x = 14, y = 5, text = "Final time: "..stringFromTime(raceTime), color = Color.White })
	window.onClose(function()
		GameObject.goToMenu()
	end)
end

GameObject.levelFailed = function ()
	gameActive = false
	levelCompleted = false
	print("Out of time!")
	local window = LuaWindow(UI.createWindow({x = 18, y = 9, width = 44, height = 8, closable = true, shadow = true, color = Color.LightRed, background = Color.Red, title = "Outta time!" }))
	UI.createLabel({parent = window.instance(), x = 17, y = 7, text = " [Return ►] ", color = Color.Yellow, background = Color.Red })
	UI.createLabel({parent = window.instance(), x = 6, y = 3, text = "You failed to complete the race..", color = Color.LightRed })
	UI.createLabel({parent = window.instance(), x = 11, y = 4, text = "Press 'Space' to retry", color = Color.LightRed })
	window.onClose(function()
		Sound.play(Music)
		GameObject.goToMenu()
	end)
	window.onKey(function(keyCode, pressed)
		if pressed then
			if keyCode == 57 then
				window.close()
				GameObject.beginGame()
				return false
			end
		end
		return true
	end)
end

GameObject.showHelp = function()
	UI.showMessageBox("Welcome to the exciting world of cycling! \nThe objective of the game is simple - finish the race before the time runs out. You can race in two modes. \n\nTake part in a sprint for a shorter contest or give your stamina a go in an endurance race. \n\nTo gain speed, consecutively and repeatedly tap the 'R' and 'L' keys or the right and left cursor keys. The faster you tap, the quicker you pedal.")
end

GameObject.cycleWallpaper = function()
	local wallpaper = Core.getWallpaper() + 1
	if wallpaper > 18 then
		wallpaper = 1
	end
	Core.setWallpaper(wallpaper)
end
